import Two from "two.js";
import { cageNum } from './design';
import { drawBg } from './background';
import { drawLight } from './effect';
import { drawWheel } from './wheel';
import { drawTargeCages } from './cage';
import { drawPedestal } from './pedestal';
import drawBullets from './bullet';
import { drawCounter } from './counter';

export default ({ el, center, rotateSpeed = 1.6, speed = -360 }) => {
    
  // 修复一个单页面应用, 不能重复加载图片的bug
  Two.Texture.ImageRegistry = new Two.Registry();

  // 初始化
  const two = new Two({
    type: Two.Types.canvas,
    fullscreen: true,
  }).appendTo(el);

  // 事件发射器
  const _evt = new Two.Events();

  // 游戏状态, 0 未开始, 1进行中, 2已结束
  let status = 0;
  const getGameState = () => status;

  // 绘制背景
  drawBg({ two, center });

  // 绘制胜利光效
  const light = drawLight({ two, center });
  
  // 目标轿厢
  const cages = drawTargeCages({ two, center })

  // 绘制摩天轮
  const wheel = drawWheel({ two, center, rotateSpeed, cages: cages.list });
  
  // 绘制底座
  const pedestal = drawPedestal({ two, center })

  // 绘制子弹计数器, 224 是设计稿上底座与计数器之间距离
  const counter = drawCounter({ two, center })

  // 绘制子弹
  const clip = drawBullets({two, center, speed, getGameState })

  // 整体绘制结束  
  two.update();

  // 下面是业务逻辑部分

  // 游戏结束
  function gameOver(isSuccess) {
    // 停止摩天轮旋转
    wheel.stop();
    status = 2; // 游戏结束

    const t = setTimeout(() => {
      clearTimeout(t)      
      _evt.dispatchEvent('gameOver', { isSuccess })
    }, 0)
  }

  // 监测子弹射击
  clip.on('beforeShoot', () => {
    pedestal.play() // 绳子甩动效果
  })
  // 监测子弹停止射击
  clip.on('afterShoot', () => {
    pedestal.stop(); // 绳子停止甩动
    clip.bulletLoad(); // 子弹上膛
    counter.value -= 1; // 计数器 - 1
  })
  // 监测子弹停止射击
  clip.on('shooting', ({bullet, cancel}) => {
    const { next, error } = cages.checkHit(bullet);
    if (next) return;

    cancel(); // 子弹停止飞行

    // 游戏失败
    if (error) {
      gameOver(false);
      return;
    }

    // 击中轿厢
    const isSuccess = !counter.value
    if (isSuccess) {
      light.start() // 启动灯光特效
      gameOver(isSuccess)
    } else {
      const t = setTimeout(() => {
        clearTimeout(t)
        _evt.dispatchEvent('bingo', { isSuccess })
      }, 0) 
    }
  })
  
  return {
    two,
    raduis: wheel.raduis,
    on: _evt.on.bind(_evt),
    fire: clip.shoot,
    play: () => {
      status = 1; // 游戏开始
      clip.bulletLoad() // 先上膛一颗子弹
      wheel.rotate();
      two.play();
    },
    pause: () => {
      wheel.stop();
      two.pause();
    },
    destroy: () => {
      two.unbind('update');
      two.pause();
      el.removeChild(two.renderer.domElement);
    }
  }
}
